#region Copyright
/*
 * Copyright 2005 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using Seasar.Controls.Win.Logic;
using Seasar.Controls.Win.Logic.Impl;

namespace Seasar.Controls.Win.Service.Impl
{
	/// <summary>
	/// XPButtonT[rX̎NXł
	/// </summary>
	internal class XPButtonServiceImpl : IXPButtonService
	{
        private static IGraphicsLogic graphicsLogic = new GraphicsLogicImpl();
        private static IXPButtonLogic xpButtonLogic = new XPButtonLogicImpl();
        
		public XPButtonServiceImpl()
		{
        }

        #region IXPButtonService o

        /// <summary>
        /// {^̏s܂
        /// </summary>
        /// <param name="button">XPButton</param>
        public void Initialize(XPButton button)
        {
            button.BackColor = SystemColors.ControlLightLight;
        }

        /// <summary>
        /// {^Text\邽߂StringFormat܂
        /// </summary>
        /// <param name="stringFormat">StringFormat</param>
        public void InitStringFormat(StringFormat stringFormat)
        {
            // Ëʒu𒆉񂹂ɂ܂
            stringFormat.Alignment = StringAlignment.Center;

            // ㉺̈ʒu𒆉񂹂ɂ܂
            stringFormat.LineAlignment = StringAlignment.Center;

            // zbgL[̂߂&A_[Cɂĕ\܂
            stringFormat.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show;
        }

        /// <summary>
        /// ʏ̏Ԃ̃{^yCg܂
        /// </summary>
        /// <param name="button">XPButton</param>
        /// <param name="graphics">GDI+ `</param>
        public void PaintNormal(XPButton button, Graphics graphics)
        {
            // NCAg̈e̔wiFœhԂ܂
            graphicsLogic.PaintSquare(graphics, button.Parent.BackColor, button.ClientSize);

            // {^̘g쐬܂
            GraphicsPath framePath = xpButtonLogic.CreateXPButtonFramePath(
                button.ClientRectangle, button.Radius);

            // {^hԂ܂
            graphics.FillPath(new SolidBrush(button.BackColor), framePath);
			
            // e̘g쐬܂
            GraphicsPath shadePath = xpButtonLogic.CreateXPButtonShadeFramePath(
                button.ClientRectangle, button.Radius);

            // e̐FhԂ܂
            graphics.FillPath(new SolidBrush(button.ShadeColor), shadePath);

            // {^̘g`܂
            graphics.DrawPath(new Pen(button.ForeColor), framePath);

            // {^̃eLXg`܂
            DrawString(button, graphics);
        }

        /// <summary>
        /// tH[JX擾Ă{^`܂
        /// </summary>
        /// <param name="button">XPButton</param>
        /// <param name="graphics">GDI+ `</param>
        public void PaintOnFocus(XPButton button, Graphics graphics)
        {
            // NCAg̈e̔wiFœhԂ܂
            graphicsLogic.PaintSquare(graphics, button.Parent.BackColor, button.ClientSize);

            // {^̘g쐬܂
            GraphicsPath framePath = xpButtonLogic.CreateXPButtonFramePath(
                button.ClientRectangle, button.Radius);

            // {^hԂ܂
            graphicsLogic.PaintGradient(graphics, button.ClientRectangle, 
                framePath, Color.AliceBlue, Color.RoyalBlue);

            // {^hԂ܂
            graphicsLogic.PaintSquare(graphics, button.BackColor,
                xpButtonLogic.GetXPButtonInsideRectangle(
                button.ClientRectangle, button.Radius));

            // {^̓ɔg`܂
            graphicsLogic.DrawSquareDotDashLine(graphics, 
                xpButtonLogic.GetXPButtonInsideRectangle(button.ClientRectangle,
                button.Radius), button.ForeColor);
			
            // e̘g쐬܂
            GraphicsPath shadePath = xpButtonLogic.CreateXPButtonShadeFramePath(
                button.ClientRectangle, button.Radius);

            // {^̘g`܂
            graphics.DrawPath(Pens.Navy, framePath);

            // {^̃eLXg`܂
            DrawString(button, graphics);
        }

        /// <summary>
        /// }EX|C^擾Ă{^`܂
        /// </summary>
        /// <param name="button">XPButton</param>
        /// <param name="graphics">GDI+ `</param>
        public void PaintOnMouseEnter(XPButton button, Graphics graphics)
        {
            // NCAg̈e̔wiFœhԂ܂
            graphicsLogic.PaintSquare(graphics, button.Parent.BackColor, button.ClientSize);

            // {^̘g쐬܂
            GraphicsPath framePath = xpButtonLogic.CreateXPButtonFramePath(
                button.ClientRectangle, button.Radius);

            // {^hԂ܂
            graphicsLogic.PaintGradient(graphics, button.ClientRectangle, 
                framePath, Color.Bisque, Color.DarkOrange);

            // {^hԂ܂
            graphicsLogic.PaintSquare(graphics, button.BackColor, 
                xpButtonLogic.GetXPButtonInsideRectangle(
                button.ClientRectangle, button.Radius));

            // tH[JX𓾂Ăꍇ́A{^̓ɔg`܂
            if((button.Modes & XPButtonModes.Focus) != XPButtonModes.None)
            {
                // {^̓ɔg`܂
                graphicsLogic.DrawSquareDotDashLine(graphics, 
                    xpButtonLogic.GetXPButtonInsideRectangle(
                    button.ClientRectangle,button.Radius), button.ForeColor);
            }
			
            // e̘g쐬܂
            GraphicsPath shadePath = xpButtonLogic.CreateXPButtonShadeFramePath(
                button.ClientRectangle, button.Radius);

            // {^̘g`܂
            graphics.DrawPath(Pens.Navy, framePath);

            // {^̃eLXg`܂
            DrawString(button, graphics);
        }

        /// <summary>
        /// }EXĂ{^`܂
        /// </summary>
        /// <param name="button">XPButton</param>
        /// <param name="graphics">GDI+ `</param>
        public void PaintOnMouseDown(XPButton button, Graphics graphics)
        {
            // NCAg̈e̔wiFœhԂ܂
            graphicsLogic.PaintSquare(graphics, button.Parent.BackColor, button.ClientSize);

            // {^̘g쐬܂
            GraphicsPath framePath = xpButtonLogic.CreateXPButtonFramePath(
                button.ClientRectangle, button.Radius);

            // {^hԂ܂
            graphics.FillPath(new SolidBrush(button.ShadeColor), framePath);

            // tH[JX𓾂Ăꍇ́A{^̓ɔg`܂
            if((button.Modes & XPButtonModes.Focus) != XPButtonModes.None)
            {
                // {^̓ɔg`܂
                graphicsLogic.DrawSquareDotDashLine(graphics, 
                    xpButtonLogic.GetXPButtonInsideRectangle(
                    button.ClientRectangle,button.Radius), button.ForeColor);
            }

            // {^̘g`܂
            graphics.DrawPath(Pens.Navy, framePath);

            // {^̃eLXg`܂
            DrawString(button, graphics);
        }

        /// <summary>
        /// wiFύXꂽꍇ̏s܂
        /// </summary>
        /// <param name="button">XPButton</param>
        public void OnBackColorChange(XPButton button)
        {
            // eF쐬Zbg܂
            button.ShadeColor = graphicsLogic.CreateShadeColor(button.BackColor);
        }

        #endregion

        #region "Private Method"

        /// <summary>
        /// XPButtonɃeLXg`܂
        /// </summary>
        /// <param name="button">XPButton</param>
        /// <param name="graphics">GDI+ `</param>
        private void DrawString(XPButton button, Graphics graphics)
        {
            graphics.DrawString(button.Text, button.Font, 
                new SolidBrush(button.ForeColor), button.ClientRectangle, button.TextFormat);
        }

        #endregion
    }
}
