#region Copyright
/*
 * Copyright 2005 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using Seasar.Controls.Win.Service;
using Seasar.Controls.Win.Service.Impl;

namespace Seasar.Controls.Win
{
	/// <summary>
	/// XPCNUIButtonł
	/// </summary>
	[System.Drawing.ToolboxBitmap(typeof(XPButton), "XPButton.bmp")]
	public class XPButton : Button
	{
        #region Private Field

        /// <summary>
        /// {^Text\邽߂StringFormatł
        /// </summary>
		private static StringFormat format = new StringFormat();

        /// <summary>
        /// XPButton̏Ԃ\܂
        /// </summary>
        private XPButtonModes modes;

        /// <summary>
        /// XPButton̊ۂݕ̔ał
        /// </summary>
		private const int RADIUS = 3;

        /// <summary>
        /// XPButton̉e̐̕Fł
        /// </summary>
		private Color shadeColor;

        /// <summary>
        /// XPButtoñT[rXł
        /// </summary>
        private static IXPButtonService xpButtonService = new XPButtonServiceImpl();

        #endregion

        #region Property

        internal StringFormat TextFormat
        {
            get { return format; }
        }

        internal int Radius
        {
            get { return RADIUS; }
        }

        internal Color ShadeColor
        {
            set { shadeColor = value; }
            get { return shadeColor; }
        }

        internal XPButtonModes Modes
        {
            get { return modes; }
        }

        #endregion

        /// <summary>
        /// XPButtoñRXgN^
        /// </summary>
		public XPButton() : base()
		{
            // XPButton̏s܂
            xpButtonService.Initialize(this);
		}

        static XPButton()
        {
            // Text\邽߂StringFormat܂
            xpButtonService.InitStringFormat(format);
        }

        #region Protected Method

		/// <summary>
		/// GotFocusCxgɌĂяo܂
		/// </summary>
		/// <param name="e"></param>
		protected override void OnGotFocus(EventArgs e)
		{
			base.OnGotFocus (e);

            // Focus𓾂Ԃǉ܂
            modes |= XPButtonModes.Focus;

            // FlatStyle.Standard̏ꍇXPButtonĕ`悵܂
			if(this.FlatStyle == FlatStyle.Standard) this.Refresh();
		}

		/// <summary>
		/// LostFocusCxgɌĂяo܂
		/// </summary>
		/// <param name="e"></param>
		protected override void OnLostFocus(EventArgs e)
		{
			base.OnLostFocus (e);

            // Focus𓾂Ԃ܂
            modes &= ~XPButtonModes.Focus;

            // FlatStyle.Standard̏ꍇXPButtonĕ`悵܂
			if(this.FlatStyle == FlatStyle.Standard) this.Refresh();
		}

		/// <summary>
		/// MouseEnterCxgɌĂяo܂
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseEnter(EventArgs e)
		{
			base.OnMouseEnter (e);

            // }EX|C^擾ĂԂǉ܂
            modes |= XPButtonModes.MouseEnter;

            // FlatStyle.Standard̏ꍇXPButtonĕ`悵܂
			if(this.FlatStyle == FlatStyle.Standard) this.Refresh();
		}

		/// <summary>
		/// MouseLeaveCxgɌĂяo܂
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseLeave(EventArgs e)
		{
			base.OnMouseLeave (e);

            // }EX|C^擾ĂԂ܂
            modes &= ~XPButtonModes.MouseEnter;

            // FlatStyle.Standard̏ꍇXPButtonĕ`悵܂
			if(this.FlatStyle == FlatStyle.Standard) this.Refresh();
		}

		/// <summary>
		/// MouseDownCxgɌĂяo܂
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);

            // }EX{^ĂԂǉ܂
            modes |= XPButtonModes.MouseDown;

            // FlatStyle.Standard̏ꍇXPButtonĕ`悵܂
			if(this.FlatStyle == FlatStyle.Standard) this.Refresh();
		}

		/// <summary>
		/// MouseUpCxgɌĂяo܂
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseUp(MouseEventArgs e)
		{
			base.OnMouseUp (e);

            // }EX{^ĂԂ܂
            modes &= ~XPButtonModes.MouseDown;

            // FlatStyle.Standard̏ꍇXPButtonĕ`悵܂
			if(this.FlatStyle == FlatStyle.Standard) this.Refresh();
		}

		/// <summary>
		/// BackColorChangedCxgɌĂяo܂
		/// </summary>
		/// <param name="e"></param>
		protected override void OnBackColorChanged(EventArgs e)
		{
			base.OnBackColorChanged (e);

            // wiFύXꂽꍇ̏s܂
			xpButtonService.OnBackColorChange(this);
		}

		/// <summary>
		/// PaintCxgɌĂяo܂
		/// </summary>
		/// <param name="pevent"></param>
		protected override void OnPaint(PaintEventArgs pevent) 
		{
            // FlatStyle.StandardȊȌꍇ́AXPX^Cɂ܂
            if(this.FlatStyle != FlatStyle.Standard)
            {
                base.OnPaint(pevent);
                return;
            }

			Graphics graphics = pevent.Graphics;

			if((modes & XPButtonModes.MouseDown) != XPButtonModes.None)
			{
				// }EXĂ{^`܂
                xpButtonService.PaintOnMouseDown(this, graphics);
			}
			else if((modes & XPButtonModes.MouseEnter) != XPButtonModes.None)
			{
				// }EX|C^擾Ă{^`܂
                xpButtonService.PaintOnMouseEnter(this, graphics);
			}
			else if((modes & XPButtonModes.Focus) != XPButtonModes.None)
			{
				// tH[JX擾Ă{^`܂
                xpButtonService.PaintOnFocus(this, graphics);
			}
			else
			{
				// ʏ̃{^`܂
				xpButtonService.PaintNormal(this, graphics);
			}
		}

		#endregion

	}
}
